;################################################################################
;#										#
;# mode 7 engine (LCD 20x2)							#
;# copyright (c) 2005-2010 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU Lesser General Public			#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# Lesser General Public License for more details.				#
;#										#
;# You should have received a copy of the GNU Lesser General Public		#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.include	"/usr/local/include/chipbasic2/m644Pdef.inc"
.include	"/usr/local/include/chipbasic2/api_macros.inc"
.include	"/usr/local/include/chipbasic2/libdef.asm"
.include	"/usr/local/include/chipbasic2/definitions.asm"

.equ		lcd_rows	= 2			;rows on LCD
.equ		lcd_cols	= 16			;columns on LCD
.equ		lcd_colbyte	= 0x80			;color for dual display
.equ		lcd_dclocks	= 35			;dummy clocks = (27-cols) * 5

.equ		lcd_charmap	= libmio_vram
.equ		lcd_sys		= libmio_vram+192
.equ		lcd_cgmap	= libmio_vram+320	;CG RAM viewed buffer
.equ		lcd_charset	= libmio_vram+256

.equ		lcd_enable	= lcd_sys		;enable update
.equ		lcd_colcount	= lcd_sys+1		;col counter
.equ		lcd_rowcount	= lcd_sys+2		;row counter
.equ		lcd_datavalid	= lcd_sys+3		;no write
.equ		lcd_databyte	= lcd_sys+4		;data to write
.equ		lcd_pixline	= lcd_sys+5		;pixline
.equ		lcd_charline	= lcd_sys+6		;charline
.equ		lcd_cgptr	= lcd_sys+8		;CG counter

.CSEG

.org	0x7200
;                   "......      N"  
start:		.db " DUAL  2*16 N",0x13,0xa1,0x20

;icon def
		.db 0xf2,0xf1,0xf1,0xf3
		.db 0xf0,"LC",0xf0
		.db 0xf4,0xf1,0xf1,0xf5

.org	0x720e
		rjmp	inex			;video, no IO
		rjmp	outex			;IO ext
		ret				;FKT 00
		rjmp	lcd216_init		;FKT 01 init
		rjmp	lcd216_cls		;FKT 02 clear screen
		rjmp	lcd216_char		;FKT 03 output char
		rjmp	lcd216_gxy		;FKT 04 set cursor
		rjmp	lcd216_plot		;FKT 05 plot
		rjmp	lcd216_newl		;FKT 06 NL
		rjmp	lcd216_vid		;FKT 07 video display
		rjmp	xmem_aread		;FKT 08 array read [7218]
		rjmp	xmem_awrite		;FKT 09 array write [7219]
		rjmp	xmem_rbyte		;FKT 0A r18=(X+) [721a]
		rjmp	xmem_wbyte		;FKT 0B (X+)=r18 [721b]
		rjmp	xmem_rword		;FKT 0C r18/r19=(X+) [721c]
		rjmp	xmem_wword		;FKT 0D (X+)=r18/r19 [721d]
		rjmp	xmem_check		;FKT 0E X=last memory cell
		rjmp	xmem_clear		;FKT 0F clear memory

;-------------------------------------------------------------------------------
; input expansion
;-------------------------------------------------------------------------------
inex:		cpi	XH,0x0a			;CG RAM
		brne	inex_err
		cpi	XL,0x40
		brcc	inex_err
		mov	r16,XL
		ldi	XL,LOW(lcd_charset)
		ldi	XH,HIGH(lcd_charset)
		add	XL,r16
		add	XH,const_0
		ld	XL,X
		clr	XH
		ret

inex_err:	ldi	ereg,40			;no IO driver
		ret

;-------------------------------------------------------------------------------
; output expansion
;-------------------------------------------------------------------------------
outex:		cpi	YH,8
		brne	outex_100

outex_10:	cpi	YL,0x00
		brne	outex_12
		sbrc	XL,0
		sbi	PORTA,0
		sbrs	XL,0
		cbi	PORTA,0
		ret
outex_12:	cpi	YL,0x01
		brne	outex_14
		sbrc	XL,0
		sbi	PORTA,1
		sbrs	XL,0
		cbi	PORTA,1
		ret
outex_14:	cpi	YL,0x02
		brne	outex_16
		sbrc	XL,0
		sbi	PORTA,2
		sbrs	XL,0
		cbi	PORTA,2
		ret
outex_16:	cpi	YL,0x03
		brne	outex_50
		sbrc	XL,0
		sbi	PORTA,3
		sbrs	XL,0
		cbi	PORTA,3
		ret

outex_50:	cpi	YL,0x10
		brne	outex_52
		sbrc	XL,0
		sbi	DDRA,0
		sbrs	XL,0
		cbi	DDRA,0
		ret
outex_52:	cpi	YL,0x11
		brne	outex_54
		sbrc	XL,0
		sbi	DDRA,1
		sbrs	XL,0
		cbi	DDRA,1
		ret
outex_54:	cpi	YL,0x12
		brne	outex_56
		sbrc	XL,0
		sbi	DDRA,2
		sbrs	XL,0
		cbi	DDRA,2
		ret
outex_56:	cpi	YL,0x13
		brne	outex_80
		sbrc	XL,0
		sbi	DDRA,3
		sbrs	XL,0
		cbi	DDRA,3
		ret

outex_80:	rjmp	outex_err

outex_100:	cpi	YH,0x0a			;CG RAM
		brne	outex_err
		cpi	YL,0x40
		brcc	outex_err
		mov	XH,YL
		ldi	YL,LOW(lcd_charset)
		ldi	YH,HIGH(lcd_charset)
		add	YL,XH
		add	YH,const_0
		st	Y,XL
		ret

outex_err:	ldi	ereg,40			;no IO driver
		ret

;-------------------------------------------------------------------------------
; init mode
;-------------------------------------------------------------------------------
lcd216_init:	sts	lcd_enable,const_0	;disable update
		sts	lcd_colcount,const_0	;col counter
		sts	lcd_rowcount,const_0	;row counter
		sts	lcd_datavalid,const_0	;no write
		sts	lcd_databyte,const_0	;data to write
		sts	lcd_cgptr,const_0	;CG counter
		brtc	lcd216_exit		;exit
		ldi	XL,0xf0
		out	DDRA,XL
		sbi	DDRB,1
		sbi	DDRB,2
		rcall	lcd_init
		rcall	lcd_ccopy
		rcall	lcd216_cls
		sts	lcd_enable,const_1	;enable update
		ldi	XL,224			;visible lines
		ret

lcd216_exit:	push	XL
		out	DDRA,const_0		;input
		cbi	DDRB,2			;busy
		pop	XL
		ret

;-------------------------------------------------------------------------------
; copy chartable
;-------------------------------------------------------------------------------
lcd_ccopy:	ldi	XL,LOW(lcd_ctable-start)
		ldi	XH,HIGH(lcd_ctable-start)
		api_dataptr
		ldi	YL,LOW(lcd_charset)
		ldi	YH,HIGH(lcd_charset)
;32 empty chars 0-31
		ldi	XL,0
lcd_ccopy_1:	st	Y+,const_0
		dec	XL
		brne	lcd_ccopy_1
;96 chars 32-127
		ldi	XH,3
		clr	XL
lcd_ccopy_2:	lpm	tempreg1,Z+
		st	Y+,tempreg1
		sbiw	XL,1
		brne	lcd_ccopy_2
;32 empty chars 128-159
		ldi	XL,0
lcd_ccopy_3:	st	Y+,const_0
		dec	XL
		brne	lcd_ccopy_3
;96 chars 160-255
		ldi	XH,3
		clr	XL
lcd_ccopy_4:	lpm	tempreg1,Z+
		st	Y+,tempreg1
		sbiw	XL,1
		brne	lcd_ccopy_4
		ret

;-------------------------------------------------------------------------------
; set cursor to position XL:XH
;-------------------------------------------------------------------------------
lcd216_gxy:	movw	libmio_cur_x,XL		;save X,Y-position
		ret

;-------------------------------------------------------------------------------
; set cursor to position XL:XH
;-------------------------------------------------------------------------------
lcd216_cls:	api_pushxyz
		ldi	YL,LOW(libmio_vram)
		ldi	YH,HIGH(libmio_vram)
		ldi	ZL,LOW(libmio_vram+256)
		ldi	ZH,HIGH(libmio_vram+256)
		ldi	XH,192
		ldi	XL,0x20
lcd216_cls_2:	st	Y+,XL
		st	Z+,const_0
		dec	XH
		brne	lcd216_cls_2
		mov	libmio_cur_x,const_0
		mov	libmio_cur_y,const_0
		api_popxyz

;-------------------------------------------------------------------------------
; output char
;-------------------------------------------------------------------------------
lcd216_char:	api_pushxyz
		movw	XL,libmio_cur_x		;get coordinates
		rcall	lcd_acalc
		inc	XL
		cpi	XL,lcd_cols
		brcs	lcd216_char1		;no X overflow
		clr	XL
		inc	XH
		cpi	XH,lcd_rows
		brcs	lcd216_char1		;no Y overflow
		clr	XH
lcd216_char1:	movw	libmio_cur_x,XL
		st	Y,tempreg1
lcd216_char2:	api_popxyz

lcd216_newl:	api_pushxyz
		movw	XL,libmio_cur_x		;get coordinates
		clr	XL
		inc	XH
		cpi	XH,lcd_rows
		brcs	lcd216_char4		;no Y overflow
		clr	XH
lcd216_char4:	movw	libmio_cur_x,XL
		api_popxyz


;-------------------------------------------------------------------------------
; plot  XL=X XH=Y
;-------------------------------------------------------------------------------
lcd216_plot:	ret

;------------------------------------------------------------------------------
; dual mode (BAS/LCD) out
;------------------------------------------------------------------------------
lcd216_vid:	mov	tempreg1,vline_l
		cpi	tempreg1,90
		brcs	lcd216_nvid
		cpi	tempreg1,128
		brcc	lcd216_nvid
		rcall	lcd_vline		;tv color line
		ret				;end of vout routine -> VS/Frame handling

lcd216_nvid:	sts	lcd_pixline,const_0	;pixline
		sts	lcd_charline,const_0	;charline
		lds	XL,lcd_enable		;enable update?
		cpi	XL,0
		breq	lcd216_nvid_1		;no update
		lds	XL,lcd_datavalid	;data to write?
		cpi	XL,1			;check
		breq	lcd216_c2		;write data
		rjmp	lcd216_c1		;prepare data
lcd216_nvid_1:	ret				;do nothing

;prepare data
lcd216_c1:	sbrc	vline_l,2		;cg/dd switch
		rjmp	lcd216_c3		;CG RAM service
;DD RAM data
		lds	XL,lcd_colcount		;col counter
		lds	XH,lcd_rowcount		;row counter
		inc	XL
		cpi	XL,lcd_cols
		brcs	lcd216_c1_10		;no X overflow
		clr	XL
		inc	XH
		cpi	XH,lcd_rows
		brcs	lcd216_c1_10		;no Y overflow
		clr	XH
lcd216_c1_10:	sts	lcd_colcount,XL		;new col counter
		sts	lcd_rowcount,XH		;new row counter
		rcall	lcd_acalc		;calc vram address
		ld	tempreg1,Y		;to write
		ld	tempreg2,Z		;visible char
		cp	tempreg1,tempreg2	;differ?
		brne	lcd216_c1_20		;yes, update
		ret

lcd216_c1_20:	st	Z,tempreg1		;set visible as written
		sts	lcd_datavalid,const_1	;set write flag
		sts	lcd_databyte,tempreg1	;data to write
		rcall	lcd_pcalc		;calculate position in LCD
		ori	tempreg1,0x80		;bit for DDRAM address
		rjmp	lcd_cout		;control output

;second cycle writes data
lcd216_c2:	sts	lcd_datavalid,const_0	;clear write flag
		lds	tempreg1,lcd_databyte	;data to write
		rjmp	lcd_dout		;data output

;CG RAM data
lcd216_c3:	lds	XL,lcd_cgptr		;cg ram counter
		inc	XL
		cpi	XL,64
		brcs	lcd216_c3_10		;no X overflow
		clr	XL
lcd216_c3_10:	sts	lcd_cgptr,XL		;new col counter
		rcall	lcd_ccalc		;calc vram address
		ld	tempreg1,Y		;to write
		ld	tempreg2,Z		;visible char
		cp	tempreg1,tempreg2	;differ?
		brne	lcd216_c3_20		;yes, update
		ret

lcd216_c3_20:	st	Z,tempreg1		;set visible as written
		sts	lcd_datavalid,const_1	;set write flag
		sts	lcd_databyte,tempreg1	;data to write
		mov	tempreg1,XL
		andi	tempreg1,0x3f		;limit to 64
		ori	tempreg1,0x40		;bit for CG RAM address
		rjmp	lcd_cout		;control output

;------------------------------------------------------------------------------
; calc address of (XL:XH) char
;------------------------------------------------------------------------------
lcd_acalc:	ldi	YL,LOW(lcd_charmap)	;VRAM address, first LCD char
		ldi	YH,HIGH(lcd_charmap)
		ldi	ZL,lcd_cols
		mul	ZL,XH
		add	YL,r0
		adc	YH,r1
		add	YL,XL
		adc	YH,const_0		;maximum 255 Bytes
		movw	ZL,YL
		inc	ZH			;visible status is addr+256
		ret

;------------------------------------------------------------------------------
; calc address of (XL) charset byte
;------------------------------------------------------------------------------
lcd_ccalc:	ldi	YL,LOW(lcd_charset)	;CGRAM address, first byte
		ldi	YH,HIGH(lcd_charset)
		add	YL,XL
		adc	YH,const_0		;maximum 64 Bytes
		movw	ZL,YL
		adiw	ZL,32			;visible status is addr+64
		adiw	ZL,32
		ret

;------------------------------------------------------------------------------
; calc LCD DD-RAM address of (XL:XH) char -> tempreg1
;------------------------------------------------------------------------------
lcd_pcalc:	ldi	ZL,64
		mul	ZL,XH
		mov	tempreg1,r0
		add	tempreg1,XL
		ret

;-------------------------------------------------------------------------------
; output character (tempreg1=char)
;-------------------------------------------------------------------------------
lcd_dout:	sbi	PORTB,2			;data
		rjmp	lcd_cout_1

;sendet ein Controlbyte an das LCD
lcd_cout:	cbi	PORTB,2			;control
lcd_cout_1:	swap	tempreg1
		rcall	lcd_outbits
		sbi	PORTB,1                 ; Enable high
		nop
		nop
		nop
		nop
		cbi	PORTB,1
		swap	tempreg1
		rcall	lcd_outbits
		sbi	PORTB,1                 ; Enable high
		nop
		nop
		nop
		nop
		cbi	PORTB,1
		ret

lcd_outbits:	sbrc	tempreg1,0
		sbi	PORTA,4
		sbrs	tempreg1,0
		cbi	PORTA,4
		sbrc	tempreg1,1
		sbi	PORTA,5
		sbrs	tempreg1,1
		cbi	PORTA,5
		sbrc	tempreg1,2
		sbi	PORTA,6
		sbrs	tempreg1,2
		cbi	PORTA,6
		sbrc	tempreg1,3
		sbi	PORTA,7
		sbrs	tempreg1,3
		cbi	PORTA,7
		ret

;enable-pulse
lcd_epulse: 	sbi	PORTB,1                 ; Enable high
		nop
		nop
		nop
		nop
		cbi	PORTB,1
		nop
		nop
		nop
		ret

;Initialisierung: muss ganz am Anfang des Programms aufgerufen werden
lcd_init:	ldi	tempreg2,10
lcd_init_1:	api_sync
		dec	tempreg2
		brne	lcd_init_1
		cbi	PORTB,2			;control
		ldi	tempreg1, 0x03		; muss 3mal hintereinander gesendet
		rcall	lcd_outbits
		rcall	lcd_epulse
		api_sync
		api_sync
		rcall	lcd_epulse
		api_sync
		api_sync
		rcall	lcd_epulse
		api_sync
		api_sync
		ldi 	tempreg1,0x02		;muss 2mal hintereinander gesendet
		rcall	lcd_outbits
		rcall	lcd_epulse
		api_sync
		api_sync
		ldi	tempreg1,0x28		;function set
		rcall	lcd_cout
		api_sync
		api_sync
		ldi	tempreg1,0x0c		;display on
		rcall	lcd_cout
		api_sync
		api_sync
		ldi	tempreg1,0x04		;no shift
		rcall	lcd_cout
		api_sync
		ret

.include	"vline.asm"
.include	"iram2.asm"

lcd_ctable:
.include	"generator/ctable.inc"
